package com.nokia.finance.tasks.jobs.car.strategy;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.config.JobConfig;
import com.nokia.finance.tasks.dao.car.CarStrategyDao;
import com.nokia.finance.tasks.utils.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * 车辆违章未处理策略定时任务
 */
@Slf4j
@Service
public class CarViolationUnprocessedJob {
    private final JobConfig jobConfig;
    private final CarStrategyDao carStrategyDao;

    CarViolationUnprocessedJob(JobConfig jobConfig, CarStrategyDao carStrategyDao) {
        this.jobConfig = jobConfig;
        this.carStrategyDao = carStrategyDao;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 2 0 15 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        log.info("执行车辆违章未处理策略定时任务");
        LocalDate localDateNow = LocalDate.now();
        LocalDate localDateEnd = localDateNow.minusMonths(1);
        Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
        try {
            boolean flag = carStrategyDao.hasCarViolationUnprocessed(endYearMonth);
            if (!flag) {
                throw new MyRuntimeException("车辆违章未处理策略缺少数据");
            }
            carStrategyDao.insertCarViolationUnprocessed(endYearMonth);
            List<LinkedHashMap<String, Object>> list = carStrategyDao.getCarViolationUnprocessed(endYearMonth);
            String dir = jobConfig.getCarViolationUnprocessedOutputPath();
            Path dirPath = Paths.get(dir);
            if (!Files.exists(dirPath)) {
                Files.createDirectories(dirPath);
                FileUtil.setDefaultDirPermissions(dirPath);
            }
            String filename = "car_violation_unprocessed_" + endYearMonth + ".csv";
            Path filePath = Paths.get(dir + filename);
            char delimiter = 1;
            try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(filePath),
                    StandardCharsets.UTF_8);
                 CSVPrinter printer = new CSVPrinter(osw,
                         CSVFormat.DEFAULT.builder().setDelimiter(delimiter).build())) {
                if (CollectionUtils.isEmpty(list)) {
                    return;
                }
                // 添加bom头避免excel乱码
                osw.write('\ufeff');
                Map<String, Object> header = list.get(0);
                // 表头
                printer.printRecord(header.keySet());
                for (Map<String, Object> map : list) {
                    printer.printRecord(map.values());
                }
            }
            FileUtil.setDefaultFilePermissions(filePath);
            Files.createDirectories(Paths.get(jobConfig.getTempOutputDir()));
            Files.copy(filePath, Paths.get(jobConfig.getTempOutputDir() + filename));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("车辆违章未处理策略定时任务失败");
        }
    }
}
